@@ -22,6 +22,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 #include <errno.h>
 #include <sys/time.h>
@@ -42,6 +43,7 @@
 #include "functions.h"
 #include "osmparser.h"
 #include "tagging.h"
+#include "uncompress.h"
 
 
 /* Global variables */
@@ -76,6 +78,7 @@ int main(int argc,char** argv)
  int         max_iterations=5;
  char       *dirname=NULL,*prefix=NULL,*tagging=NULL,*errorlog=NULL;
  int         option_parse_only=0,option_process_only=0;
+ int         option_append=0,option_keep=0,option_changes=0;
  int         option_filenames=0;
  int         option_prune_isolated=500,option_prune_short=5,option_prune_straight=3;
  int         arg;
@@ -88,22 +91,20 @@ int main(int argc,char** argv)
    {
     if(!strcmp(argv[arg],"--help"))
        print_usage(1,NULL,NULL);
+    else if(!strncmp(argv[arg],"--dir=",6))
+       dirname=&argv[arg][6];
+    else if(!strncmp(argv[arg],"--prefix=",9))
+       prefix=&argv[arg][9];
     else if(!strncmp(argv[arg],"--sort-ram-size=",16))
        option_filesort_ramsize=atoi(&argv[arg][16]);
 #if defined(USE_PTHREADS) && USE_PTHREADS
     else if(!strncmp(argv[arg],"--sort-threads=",15))
        option_filesort_threads=atoi(&argv[arg][15]);
 #endif
-    else if(!strncmp(argv[arg],"--dir=",6))
-       dirname=&argv[arg][6];
     else if(!strncmp(argv[arg],"--tmpdir=",9))
        option_tmpdirname=&argv[arg][9];
-    else if(!strncmp(argv[arg],"--prefix=",9))
-       prefix=&argv[arg][9];
-    else if(!strcmp(argv[arg],"--parse-only"))
-       option_parse_only=1;
-    else if(!strcmp(argv[arg],"--process-only"))
-       option_process_only=1;
+    else if(!strncmp(argv[arg],"--tagging=",10))
+       tagging=&argv[arg][10];
     else if(!strcmp(argv[arg],"--loggable"))
        option_loggable=1;
     else if(!strcmp(argv[arg],"--logtime"))
@@ -112,10 +113,18 @@ int main(int argc,char** argv)
        errorlog="error.log";
     else if(!strncmp(argv[arg],"--errorlog=",11))
        errorlog=&argv[arg][11];
+    else if(!strcmp(argv[arg],"--parse-only"))
+       option_parse_only=1;
+    else if(!strcmp(argv[arg],"--process-only"))
+       option_process_only=1;
+    else if(!strcmp(argv[arg],"--append"))
+       option_append=1;
+    else if(!strcmp(argv[arg],"--keep"))
+       option_keep=1;
+    else if(!strcmp(argv[arg],"--changes"))
+       option_changes=1;
     else if(!strncmp(argv[arg],"--max-iterations=",17))
        max_iterations=atoi(&argv[arg][17]);
-    else if(!strncmp(argv[arg],"--tagging=",10))
-       tagging=&argv[arg][10];
     else if(!strncmp(argv[arg],"--prune",7))
       {
        if(!strcmp(&argv[arg][7],"-none"))
@@ -140,9 +149,15 @@ int main(int argc,char** argv)
  if(option_parse_only && option_process_only)
     print_usage(0,NULL,"Cannot use '--parse-only' and '--process-only' at the same time.");
 
+ if(option_append && option_process_only)
+    print_usage(0,NULL,"Cannot use '--append' and '--process-only' at the same time.");
+
  if(option_filenames && option_process_only)
     print_usage(0,NULL,"Cannot use '--process-only' and filenames at the same time.");
 
+ if(!option_filenames && !option_process_only)
+    print_usage(0,NULL,"File names must be specified unless using '--process-only'");
+
  if(!option_filesort_ramsize)
    {
 #if SLIM
@@ -162,7 +177,6 @@ int main(int argc,char** argv)
        option_tmpdirname=dirname;
    }
 
- 
  if(!option_process_only)
    {
     if(tagging)
@@ -195,61 +209,102 @@ int main(int argc,char** argv)
 
  /* Create new node, segment, way and relation variables */
 
- Nodes=NewNodeList(option_parse_only||option_process_only);
+ Nodes=NewNodeList(option_append||option_changes,option_process_only);
 
- Segments=NewSegmentList(option_parse_only||option_process_only);
+ Segments=NewSegmentList(option_append||option_changes,option_process_only);
 
- Ways=NewWayList(option_parse_only||option_process_only);
+ Ways=NewWayList(option_append||option_changes,option_process_only);
 
- Relations=NewRelationList(option_parse_only||option_process_only);
+ Relations=NewRelationList(option_append||option_changes,option_process_only);
 
  /* Create the error log file */
 
  if(errorlog)
-    open_errorlog(FileName(dirname,prefix,errorlog),option_parse_only||option_process_only);
+    open_errorlog(FileName(dirname,prefix,errorlog),option_append||option_changes||option_process_only);
 
  /* Parse the file */
 
 if(!option_process_only)
   {
-   if(option_filenames)
+   for(arg=1;arg<argc;arg++)
      {
-      for(arg=1;arg<argc;arg++)
+      int fd;
+      char *filename,*p;
+
+      if(argv[arg][0]=='-' && argv[arg][1]=='-')
+         continue;
+
+      filename=strcpy(malloc(strlen(argv[arg])+1),argv[arg]);
+
+      fd=ReOpenFile(filename);
+
+      if((p=strstr(filename,".bz2")) && !strcmp(p,".bz2"))
         {
-         FILE *file;
+         fd=Uncompress_Bzip2(fd);
+         *p=0;
+        }
 
-         if(argv[arg][0]=='-' && argv[arg][1]=='-')
-            continue;
+      if((p=strstr(filename,".gz")) && !strcmp(p,".gz"))
+        {
+         fd=Uncompress_Gzip(fd);
+         *p=0;
+        }
 
-         file=fopen(argv[arg],"rb");
-
-         if(!file)
-           {
-            fprintf(stderr,"Cannot open file '%s' for reading [%s].\n",argv[arg],strerror(errno));
-            exit(EXIT_FAILURE);
-           }
-
-         printf("\nParse OSM Data [%s]\n==============\n\n",argv[arg]);
+      if(option_changes)
+        {
+         printf("\nParse OSC Data [%s]\n==============\n\n",filename);
          fflush(stdout);
 
-         if(ParseOSM(file,Nodes,Segments,Ways,Relations))
-            exit(EXIT_FAILURE);
-
-         fclose(file);
+         if((p=strstr(filename,".pbf")) && !strcmp(p,".pbf"))
+           {
+            logassert(0,"Unable to read a PBF file to apply changes (format does not permit this)");
+           }
+         else if((p=strstr(filename,".o5c")) && !strcmp(p,".o5c"))
+           {
+            if(ParseO5CFile(fd,Nodes,Segments,Ways,Relations))
+               exit(EXIT_FAILURE);
+           }
+         else
+           {
+            if(ParseOSCFile(fd,Nodes,Segments,Ways,Relations))
+               exit(EXIT_FAILURE);
+           }
         }
-     }
-   else
-     {
-      printf("\nParse OSM Data\n==============\n\n");
-      fflush(stdout);
+      else
+        {
+         printf("\nParse OSM Data [%s]\n==============\n\n",filename);
+         fflush(stdout);
 
-      if(ParseOSM(stdin,Nodes,Segments,Ways,Relations))
-         exit(EXIT_FAILURE);
+         if((p=strstr(filename,".pbf")) && !strcmp(p,".pbf"))
+           {
+            if(ParsePBFFile(fd,Nodes,Segments,Ways,Relations))
+               exit(EXIT_FAILURE);
+           }
+         else if((p=strstr(filename,".o5m")) && !strcmp(p,".o5m"))
+           {
+            if(ParseO5MFile(fd,Nodes,Segments,Ways,Relations))
+               exit(EXIT_FAILURE);
+           }
+         else
+           {
+            if(ParseOSMFile(fd,Nodes,Segments,Ways,Relations))
+               exit(EXIT_FAILURE);
+           }
+        }
+
+      CloseFile(fd);
+
+      free(filename);
      }
 
    DeleteXMLTaggingRules();
   }
 
+ FinishNodeList(Nodes);
+ FinishSegmentList(Segments);
+ FinishWayList(Ways);
+ FinishRelationList(Relations);
+
  if(option_parse_only)
    {
     FreeNodeList(Nodes,1);
@@ -260,38 +315,47 @@ if(!option_process_only)
     return(0);
    }
 
- /* Process the data */
 
- printf("\nProcess OSM Data\n================\n\n");
+ /* Sort the data */
+
+ printf("\nSort OSM Data\n=============\n\n");
  fflush(stdout);
 
  /* Sort the nodes, segments, ways and relations */
 
  SortNodeList(Nodes);
 
- SortSegmentList(Segments,0);
+ if(option_changes)
+    ApplySegmentChanges(Segments);
+
+ SortSegmentList(Segments);
 
  SortWayList(Ways);
 
  SortRelationList(Relations);
 
- /* Remove bad segments (must be after sorting the nodes and segments) */
+ /* Process the data */
 
- RemoveBadSegments(Nodes,Segments);
+ printf("\nProcess OSM Data\n================\n\n");
+ fflush(stdout);
+
+ /* Extract the way names (must be before using the ways) */
+
+ ExtractWayNames(Ways,option_keep||option_changes);
+
+ /* Remove bad segments (must be after sorting the nodes, segments and ways) */
+
+ RemoveBadSegments(Segments,Nodes,Ways,option_keep||option_changes);
 
  /* Remove non-highway nodes (must be after removing the bad segments) */
 
- RemoveNonHighwayNodes(Nodes,Segments);
+ RemoveNonHighwayNodes(Nodes,Segments,option_keep||option_changes);
 
  /* Process the route relations and first part of turn relations (must be before compacting the ways) */
 
- ProcessRouteRelations(Relations,Ways);
+ ProcessRouteRelations(Relations,Ways,option_keep||option_changes);
 
- ProcessTurnRelations1(Relations,Nodes,Ways);
-
- /* Compact the ways (must be before measuring the segments) */
-
- CompactWayList(Ways);
+ ProcessTurnRelations1(Relations,Nodes,Ways,option_keep||option_changes);
 
  /* Measure the segments and replace node/way id with index (must be after removing non-highway nodes) */
 
@@ -299,16 +363,27 @@ if(!option_process_only)
 
  /* Index the segments */
 
- IndexSegments(Segments,Nodes);
+ IndexSegments(Segments,Nodes,Ways);
 
  /* Convert the turn relations from ways into nodes */
 
  ProcessTurnRelations2(Relations,Nodes,Segments,Ways);
 
+ /* Compact the ways (must be after turn relations 2) */
+
+ CompactWayList(Ways,Segments);
+
+ /* Index the segments */
+
+ IndexSegments(Segments,Nodes,Ways);
+
  /* Prune unwanted nodes/segments. */
 
  if(option_prune_straight || option_prune_isolated || option_prune_short)
    {
+    printf("\nPrune Unneeded Data\n===================\n\n");
+    fflush(stdout);
+
     StartPruning(Nodes,Segments,Ways);
 
     if(option_prune_straight)
@@ -321,6 +396,14 @@ if(!option_process_only)
        PruneShortSegments(Nodes,Segments,Ways,option_prune_short);
 
     FinishPruning(Nodes,Segments,Ways);
+
+    /* Remove the pruned nodes and segments and update the indexes */
+
+    RemovePrunedNodes(Nodes,Segments);
+    RemovePrunedSegments(Segments,Ways);
+    CompactWayList(Ways,Segments);
+    RemovePrunedTurnRelations(Relations,Nodes);
+    IndexSegments(Segments,Nodes,Ways);
    }
 
  /* Repeated iteration on Super-Nodes and Super-Segments */
@@ -363,17 +446,13 @@ if(!option_process_only)
        SuperSegments=SuperSegments2;
       }
 
-    /* Sort the super-segments */
+    /* Sort the super-segments and remove duplicates */
 
-    SortSegmentList(SuperSegments,0);
-
-    /* Remove duplicated super-segments */
-
-    DeduplicateSegments(SuperSegments,Nodes,Ways);
+    DeduplicateSuperSegments(SuperSegments,Ways);
 
     /* Index the segments */
 
-    IndexSegments(SuperSegments,Nodes);
+    IndexSegments(SuperSegments,Nodes,Ways);
 
     /* Check for end condition */
 
@@ -402,38 +481,28 @@ if(!option_process_only)
 
  Segments=MergedSegments;
 
- /* Sort and re-index the segments */
+ /* Re-index the merged segments */
 
- SortSegmentList(Segments,0);
-
- IndexSegments(Segments,Nodes);
+ IndexSegments(Segments,Nodes,Ways);
 
  /* Cross reference the nodes and segments */
 
  printf("\nCross-Reference Nodes and Segments\n==================================\n\n");
  fflush(stdout);
 
- /* Sort the nodes geographically and update the segment indexes accordingly */
+ /* Sort the nodes and segments geographically */
 
  SortNodeListGeographically(Nodes);
 
- UpdateSegments(Segments,Nodes,Ways);
+ SortSegmentListGeographically(Segments,Nodes);
 
- /* Sort the segments geographically and re-index them */
+ /* Re-index the segments */
 
- SortSegmentList(Segments,0);
+ IndexSegments(Segments,Nodes,Ways);
 
- IndexSegments(Segments,Nodes);
+ /* Sort the turn relations geographically */
 
- /* Update the nodes */
-
- UpdateNodes(Nodes,Segments);
-
- /* Fix the turn relations after sorting nodes geographically */
-
- UpdateTurnRelations(Relations,Nodes,Segments);
-
- SortTurnRelationList(Relations);
+ SortTurnRelationListGeographically(Relations,Nodes,Segments);
 
  /* Output the results */
 
@@ -442,7 +511,7 @@ if(!option_process_only)
 
  /* Write out the nodes */
 
- SaveNodeList(Nodes,FileName(dirname,prefix,"nodes.mem"));
+ SaveNodeList(Nodes,FileName(dirname,prefix,"nodes.mem"),Segments);
 
  FreeNodeList(Nodes,0);
 
@@ -508,12 +577,22 @@ static void print_usage(int detail,const char *argerr,const char *err)
          "                      [--loggable] [--logtime]\n"
          "                      [--errorlog[=<name>]]\n"
          "                      [--parse-only | --process-only]\n"
+         "                      [--append] [--keep] [--changes]\n"
          "                      [--max-iterations=<number>]\n"
          "                      [--prune-none]\n"
          "                      [--prune-isolated=<len>]\n"
          "                      [--prune-short=<len>]\n"
          "                      [--prune-straight=<len>]\n"
-         "                      [<filename.osm> ...]\n");
+         "                      [<filename.osm> ... | <filename.osc> ...\n"
+         "                       | <filename.pbf> ...\n"
+         "                       | <filename.osm> ... | <filename.osc> ..."
+#if defined(USE_BZIP2) && USE_BZIP2
+         "\n                       | <filename.(osm|osc|o5m|o5c).bz2> ..."
+#endif
+#if defined(USE_GZIP) && USE_GZIP
+         "\n                       | <filename.(osm|osc|o5m|o5c).gz> ..."
+#endif
+         "]\n");
 
  if(argerr)
     fprintf(stderr,
@@ -556,23 +635,34 @@ static void print_usage(int detail,const char *argerr,const char *err)
             "--errorlog[=<name>]       Log parsing errors to 'error.log' or the given name\n"
             "                          (the '--dir' and '--prefix' options are applied).\n"
             "\n"
-            "--parse-only              Parse the input OSM files and store the results.\n"
+            "--parse-only              Parse the OSM/OSC file(s) and store the results.\n"
             "--process-only            Process the stored results from previous option.\n"
+            "--append                  Parse the OSM file(s) and append to existing results.\n"
+            "--keep                    Keep the intermediate files after parsing & sorting.\n"
+            "--changes                 Parse the data as an OSC file and apply the changes.\n"
             "\n"
             "--max-iterations=<number> The number of iterations for finding super-nodes\n"
             "                          (defaults to 5).\n"
             "\n"
             "--prune-none              Disable the prune options below, they are re-enabled\n"
             "                          by adding them to the command line after this option.\n"
-            "--prune-isolated=<len>    Remove small disconnected groups of segments\n"
+            "--prune-isolated=<len>    Remove access from small disconnected segment groups\n"
             "                          (defaults to removing groups under 500m).\n"
             "--prune-short=<len>       Remove short segments (defaults to removing segments\n"
             "                          up to a maximum length of 5m).\n"
             "--prune-straight=<len>    Remove nodes in almost straight highways (defaults to\n"
             "                          removing nodes up to 3m offset from a straight line).\n"
             "\n"
-            "<filename.osm> ...        The name(s) of the file(s) to process (by default\n"
-            "                          data is read from standard input).\n"
+            "<filename.osm>, <filename.osc>, <filename.pbf>, <filename.o5m>, <filename.o5c>\n"
+            "                          The name(s) of the file(s) to read and parse.\n"
+            "                          Filenames ending '.pbf' read as PBF, filenames ending\n"
+            "                          '.o5m' or '.o5c' read as O5M/O5C, others as XML.\n"
+#if defined(USE_BZIP2) && USE_BZIP2
+            "                          Filenames ending '.bz2' will be bzip2 uncompressed.\n"
+#endif
+#if defined(USE_GZIP) && USE_GZIP
+            "                          Filenames ending '.gz' will be gzip uncompressed.\n"
+#endif
             "\n"
             "<transport> defaults to all but can be set to:\n"
             "%s"
