@@ -67,6 +67,9 @@ void StartPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
  SegmentX segmentx;
  index_t index=0,lastnode1=NO_NODE;
 
+ if(segmentsx->number==0)
+    return;
+
  /* Print the start message */
 
  printf_first("Adding Extra Segment Indexes: Segments=0");
@@ -125,7 +128,9 @@ void StartPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
 void FinishPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 {
- free(segmentsx->next1);
+ if(segmentsx->next1)
+    free(segmentsx->next1);
+
  segmentsx->next1=NULL;
 }
 
@@ -848,7 +853,7 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
  logassert(lats,"Failed to allocate memory (try using slim mode?)");    /* Check malloc() worked */
  logassert(lons,"Failed to allocate memory (try using slim mode?)");    /* Check malloc() worked */
 
- /* Loop through the nodes and find stretchs of simple highway for possible modification */
+ /* Loop through the nodes and find stretches of simple highway for possible modification */
 
  for(i=0;i<nodesx->number;i++)
    {
@@ -894,7 +899,7 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
 
           /* Count the segments connected to the node */
 
-          segmentx=FirstSegmentX(segmentsx,nodes[current],1);
+          segmentx=FirstSegmentX(segmentsx,nodes[current],3);
 
           while(segmentx)
             {
@@ -968,6 +973,8 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
 
        if(segcount==2)
          {
+          /* Make space in the lists */
+
           if(upper==(nalloc-1))
             {
              nodes   =(index_t*)realloc(nodes   ,(nalloc+=1024)*sizeof(index_t));
@@ -1045,7 +1052,7 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
              upperbounded=1;
             }
          }
-       else
+       else /* if(segment!=2) */
          {
           if(current==upper)
              upperbounded=1;
