@@ -602,7 +602,7 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx,int keep)
                 ways++;
                }
              else
-                logerror("Route Relation %"Prelation_t" contains Way %"Pway_t" but it does not exist in the Routino database.\n",relationx.id,wayid);
+                logerror("Route Relation %"Prelation_t" contains Way %"Pway_t" that does not exist in the Routino database (not a highway?).\n",relationx.id,wayid);
             }
          }
        while(wayid!=NO_WAY_ID);
@@ -718,13 +718,13 @@ void ProcessTurnRelations1(RelationsX *relationsx,NodesX *nodesx,WaysX *waysx,in
     to  =IndexWayX(waysx,relationx.to);
 
     if(via==NO_NODE)
-       logerror("Turn Relation %"Prelation_t" contains Node %"Pnode_t" but it does not exist in the Routino database.\n",relationx.id,relationx.via);
+       logerror("Turn Relation %"Prelation_t" contains Node %"Pnode_t" that does not exist in the Routino database (not a highway node?).\n",relationx.id,relationx.via);
 
     if(from==NO_WAY)
-       logerror("Turn Relation %"Prelation_t" contains Way %"Pway_t" but it does not exist in the Routino database.\n",relationx.id,relationx.from);
+       logerror("Turn Relation %"Prelation_t" contains Way %"Pway_t" that does not exist in the Routino database (not a highway?).\n",relationx.id,relationx.from);
 
     if(to==NO_WAY)
-       logerror("Turn Relation %"Prelation_t" contains Way %"Pway_t" but it does not exist in the Routino database.\n",relationx.id,relationx.to);
+       logerror("Turn Relation %"Prelation_t" contains Way %"Pway_t" that does not exist in the Routino database (not a highway?).\n",relationx.id,relationx.to);
 
     relationx.via =via;
     relationx.from=from;
@@ -834,7 +834,7 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
              if(IsOnewayFrom(segmentx,relationx.via))
                 oneway_from=1;  /* not allowed */
 
-             if(!(wayx->way.allow&(Transports_Bicycle|Transports_Moped|Transports_Motorbike|Transports_Motorcar|Transports_Goods|Transports_HGV|Transports_PSV)))
+             if(!(wayx->way.allow&(Transports_Bicycle|Transports_Moped|Transports_Motorcycle|Transports_Motorcar|Transports_Goods|Transports_HGV|Transports_PSV)))
                 vehicles_from=0;  /* not allowed */
             }
 
@@ -854,7 +854,7 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
              if(IsOnewayTo(segmentx,relationx.via))
                 oneway_to=1;  /* not allowed */
 
-             if(!(wayx->way.allow&(Transports_Bicycle|Transports_Moped|Transports_Motorbike|Transports_Motorcar|Transports_Goods|Transports_HGV|Transports_PSV)))
+             if(!(wayx->way.allow&(Transports_Bicycle|Transports_Moped|Transports_Motorcycle|Transports_Motorcar|Transports_Goods|Transports_HGV|Transports_PSV)))
                 vehicles_to=0;  /* not allowed */
             }
 
@@ -925,7 +925,7 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
              if(IsOnewayFrom(segmentx,relationx.via))
                 oneway_from=1;  /* not allowed */
 
-             if(!(wayx->way.allow&(Transports_Bicycle|Transports_Moped|Transports_Motorbike|Transports_Motorcar|Transports_Goods|Transports_HGV|Transports_PSV)))
+             if(!(wayx->way.allow&(Transports_Bicycle|Transports_Moped|Transports_Motorcycle|Transports_Motorcar|Transports_Goods|Transports_HGV|Transports_PSV)))
                 vehicles_from=0;  /* not allowed */
             }
 
@@ -947,7 +947,7 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
 
              if(IsOnewayTo(segmentx,relationx.via))
                 ;  /* not allowed */
-             else if(!(wayx->way.allow&(Transports_Bicycle|Transports_Moped|Transports_Motorbike|Transports_Motorcar|Transports_Goods|Transports_HGV|Transports_PSV)))
+             else if(!(wayx->way.allow&(Transports_Bicycle|Transports_Moped|Transports_Motorcycle|Transports_Motorcar|Transports_Goods|Transports_HGV|Transports_PSV)))
                 ;  /* not allowed */
              else
                {
@@ -1058,6 +1058,9 @@ void RemovePrunedTurnRelations(RelationsX *relationsx,NodesX *nodesx)
  index_t total=0,pruned=0,notpruned=0;
  int trfd;
 
+ if(relationsx->trnumber==0)
+    return;
+
  /* Print the start message */
 
  printf_first("Deleting Pruned Turn Relations: Relations=0 Pruned=0");
@@ -1120,6 +1123,9 @@ void SortTurnRelationListGeographically(RelationsX *relationsx,NodesX *nodesx,Se
 {
  int trfd;
 
+ if(segmentsx->number==0)
+    return;
+
  /* Print the start message */
 
  printf_first("Sorting Turn Relations Geographically");
