@@ -530,7 +530,7 @@ static int deduplicate(SegmentX *segmentx,index_t index)
 
 void RemoveBadSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx,int keep)
 {
- index_t noway=0,loop=0,nonode=0,duplicate=0,good=0,total=0;
+ index_t noway=0,nonode=0,duplicate=0,good=0,total=0;
  node_t prevnode1=NO_NODE_ID,prevnode2=NO_NODE_ID;
  way_t prevway=NO_WAY_ID;
  distance_t prevdist=0;
@@ -539,7 +539,7 @@ void RemoveBadSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx,int keep
 
  /* Print the start message */
 
- printf_first("Checking Segments: Segments=0 Loop=0 No-Way=0 No-Node=0 Duplicate=0");
+ printf_first("Checking Segments: Segments=0 No-Way=0 No-Node=0 Duplicate=0");
 
  /* Allocate the node usage bitmask */
 
@@ -568,26 +568,20 @@ void RemoveBadSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx,int keep
 
     if(indexw==NO_WAY)
       {
-       logerror("Segment belongs to way %"Pway_t" but it doesn't exist.\n",segmentx.way);
+       logerror("Segment belongs to way %"Pway_t" that does not exist in the Routino database (not a highway?).\n",segmentx.way);
 
        noway++;
       }
-    else if(segmentx.node1==segmentx.node2)
-      {
-       logerror("Segment connects node %"Pnode_t" to itself.\n",segmentx.node1);
-
-       loop++;
-      }
     else if(index1==NO_NODE || index2==NO_NODE)
       {
        if(index1==NO_NODE && index2==NO_NODE)
-          logerror("Segment connects nodes %"Pnode_t" and %"Pnode_t" but neither exist.\n",segmentx.node1,segmentx.node2);
+          logerror("Segment connects nodes %"Pnode_t" and %"Pnode_t" that do not exist in the Routino database (not highway nodes?).\n",segmentx.node1,segmentx.node2);
 
        if(index1==NO_NODE && index2!=NO_NODE)
-          logerror("Segment connects nodes %"Pnode_t" and %"Pnode_t" but the first one does not exist.\n",segmentx.node1,segmentx.node2);
+          logerror("Segment contains node %"Pnode_t" that does not exist in the Routino database (not a highway node?).\n",segmentx.node1);
 
        if(index1!=NO_NODE && index2==NO_NODE)
-          logerror("Segment connects nodes %"Pnode_t" and %"Pnode_t" but the second one does not exist.\n",segmentx.node1,segmentx.node2);
+          logerror("Segment contains node %"Pnode_t" that does not exist in the Routino database (not a highway node?).\n",segmentx.node2);
 
        nonode++;
       }
@@ -630,7 +624,7 @@ void RemoveBadSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx,int keep
     total++;
 
     if(!(total%10000))
-       printf_middle("Checking Segments: Segments=%"Pindex_t" Loop=%"Pindex_t" No-Way=%"Pindex_t" No-Node=%"Pindex_t" Duplicate=%"Pindex_t,total,loop,noway,nonode,duplicate);
+       printf_middle("Checking Segments: Segments=%"Pindex_t" No-Way=%"Pindex_t" No-Node=%"Pindex_t" Duplicate=%"Pindex_t,total,noway,nonode,duplicate);
    }
 
  segmentsx->number=good;
@@ -642,7 +636,7 @@ void RemoveBadSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx,int keep
 
  /* Print the final message */
 
- printf_last("Checked Segments: Segments=%"Pindex_t" Loop=%"Pindex_t" No-Way=%"Pindex_t" No-Node=%"Pindex_t" Duplicate=%"Pindex_t,total,loop,noway,nonode,duplicate);
+ printf_last("Checked Segments: Segments=%"Pindex_t" No-Way=%"Pindex_t" No-Node=%"Pindex_t" Duplicate=%"Pindex_t,total,noway,nonode,duplicate);
 }
 
 
@@ -662,6 +656,9 @@ void MeasureSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
  int fd;
  SegmentX segmentx;
 
+ if(segmentsx->number==0)
+    return;
+
  /* Print the start message */
 
  printf_first("Measuring Segments: Segments=0");
@@ -857,6 +854,9 @@ void RemovePrunedSegments(SegmentsX *segmentsx,WaysX *waysx)
  int fd;
  index_t xnumber;
 
+ if(segmentsx->number==0)
+    return;
+
  /* Print the start message */
 
  printf_first("Sorting and Pruning Segments");
@@ -930,6 +930,9 @@ void DeduplicateSuperSegments(SegmentsX *segmentsx,WaysX *waysx)
  int fd;
  index_t xnumber;
 
+ if(waysx->number==0)
+    return;
+
  /* Print the start message */
 
  printf_first("Sorting and Deduplicating Super-Segments");
@@ -1059,6 +1062,9 @@ void SortSegmentListGeographically(SegmentsX *segmentsx,NodesX *nodesx)
 {
  int fd;
 
+ if(segmentsx->number==0)
+    return;
+
  /* Print the start message */
 
  printf_first("Sorting Segments Geographically");
