@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2010-2011 Andrew M. Bishop
+ This file Copyright 2010-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -22,39 +22,34 @@
 #ifndef TAGGING_H
 #define TAGGING_H    /*+ To stop multiple inclusions. +*/
 
+#include <stdint.h>
+
 
 /* Data types */
 
-/*+ A structure to contain the tagging action. +*/
-typedef struct _TaggingAction
-{
- int output;                    /*+ A flag to indicate if the output tags or input tags are to be changed. +*/
-
- char *k;                       /*+ The tag key (or NULL). +*/
- char *v;                       /*+ The tag value (or NULL). +*/
-}
- TaggingAction;
+typedef struct _TaggingRuleList TaggingRuleList;
 
 
-/*+ A structure to contain the tagging rule. +*/
+/*+ A structure to contain the tagging rule/action. +*/
 typedef struct _TaggingRule
 {
+ int action;                    /*+ A flag to indicate the type of action. +*/
+
  char *k;                       /*+ The tag key (or NULL). +*/
  char *v;                       /*+ The tag value (or NULL). +*/
+ char *message;                 /*+ The message string for logerror (or NULL). +*/
 
- TaggingAction *actions;        /*+ The actions to take. +*/
- int            nactions;       /*+ The number of actions. +*/
+ TaggingRuleList *rulelist;     /*+ The sub-rules belonging to this rule. +*/
 }
  TaggingRule;
 
 
 /*+ A structure to contain the list of rules and associated information. +*/
-typedef struct _TaggingRuleList
+struct _TaggingRuleList
 {
  TaggingRule *rules;            /*+ The array of rules. +*/
  int          nrules;           /*+ The number of rules. +*/
-}
- TaggingRuleList;
+};
 
 
 /*+ A structure to hold a list of tags to be processed. +*/
@@ -68,28 +63,19 @@ typedef struct _TagList
  TagList;
 
 
-/* Global variables */
-
-extern TaggingRuleList NodeRules;
-extern TaggingRuleList WayRules;
-extern TaggingRuleList RelationRules;
-
-
 /* Functions in tagging.c */
 
 int ParseXMLTaggingRules(const char *filename);
 void DeleteXMLTaggingRules(void);
 
-TaggingRule *AppendTaggingRule(TaggingRuleList *rules,const char *k,const char *v);
-void AppendTaggingAction(TaggingRule *rule,const char *k,const char *v,int output);
-void DeleteTaggingRuleList(TaggingRuleList *rules);
-
 TagList *NewTagList(void);
-void AppendTag(TagList *tags,const char *k,const char *v);
-void ModifyTag(TagList *tags,const char *k,const char *v);
 void DeleteTagList(TagList *tags);
 
-TagList *ApplyTaggingRules(TaggingRuleList *rules,TagList *tags);
+void AppendTag(TagList *tags,const char *k,const char *v);
+
+TagList *ApplyNodeTaggingRules(TagList *tags,int64_t id);
+TagList *ApplyWayTaggingRules(TagList *tags,int64_t id);
+TagList *ApplyRelationTaggingRules(TagList *tags,int64_t id);
 
 
 #endif /* TAGGING_H */
