@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2010-2011 Andrew M. Bishop
+ This file Copyright 2010-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -22,28 +22,30 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 #include <ctype.h>
-#include <errno.h>
+#include <inttypes.h>
+#include <stdint.h>
+
+#include "xmlparse.h"
+#include "logging.h"
+#include "tagging.h"
 
 #include "files.h"
-#include "logging.h"
-#include "xmlparse.h"
-#include "tagging.h"
+#include "uncompress.h"
 
 
 /* Local variables */
 
-static unsigned long nnodes=0;
-static unsigned long nways=0;
-static unsigned long nrelations=0;
+static uint64_t nnodes=0,nways=0,nrelations=0;
 
 TagList *current_tags=NULL;
 
 
 /* Local functions */
 
-static void print_usage(int detail);
+static void print_usage(int detail,const char *argerr,const char *err);
 
 
 /* The XML tag processing function prototypes */
@@ -255,19 +257,25 @@ static int tagType_function(const char *_tag_,int _type_,const char *k,const cha
 
 static int nodeType_function(const char *_tag_,int _type_,const char *id,const char *lat,const char *lon,const char *timestamp,const char *uid,const char *user,const char *visible,const char *version,const char *action)
 {
+ static int64_t llid;
+
  if(_type_&XMLPARSE_TAG_START)
    {
     nnodes++;
 
     if(!(nnodes%10000))
-       fprintf_middle(stderr,"Reading: Lines=%llu Nodes=%lu Ways=%lu Relations=%lu",ParseXML_LineNumber(),nnodes,nways,nrelations);
+       fprintf_middle(stderr,"Reading: Lines=%"PRIu64" Nodes=%"PRIu64" Ways=%"PRIu64" Relations=%"PRIu64,ParseXML_LineNumber(),nnodes,nways,nrelations);
 
     current_tags=NewTagList();
+
+    /* Handle the node information */
+
+    XMLPARSE_ASSERT_INTEGER(_tag_,id); llid=atoll(id); /* need int64_t conversion */
    }
 
  if(_type_&XMLPARSE_TAG_END)
    {
-    TagList *result=ApplyTaggingRules(&NodeRules,current_tags);
+    TagList *result=ApplyNodeTaggingRules(current_tags,llid);
     int i;
 
     for(i=0;i<result->ntags;i++)
@@ -371,19 +379,25 @@ static int memberType_function(const char *_tag_,int _type_,const char *type,con
 
 static int wayType_function(const char *_tag_,int _type_,const char *id,const char *timestamp,const char *uid,const char *user,const char *visible,const char *version,const char *action)
 {
+ static int64_t llid;
+
  if(_type_&XMLPARSE_TAG_START)
    {
     nways++;
 
     if(!(nways%1000))
-       fprintf_middle(stderr,"Reading: Lines=%llu Nodes=%lu Ways=%lu Relations=%lu",ParseXML_LineNumber(),nnodes,nways,nrelations);
+       fprintf_middle(stderr,"Reading: Lines=%"PRIu64" Nodes=%"PRIu64" Ways=%"PRIu64" Relations=%"PRIu64,ParseXML_LineNumber(),nnodes,nways,nrelations);
 
     current_tags=NewTagList();
+
+    /* Handle the way information */
+
+    XMLPARSE_ASSERT_INTEGER(_tag_,id); llid=atoll(id); /* need int64_t conversion */
    }
 
  if(_type_&XMLPARSE_TAG_END)
    {
-    TagList *result=ApplyTaggingRules(&WayRules,current_tags);
+    TagList *result=ApplyWayTaggingRules(current_tags,llid);
     int i;
 
     for(i=0;i<result->ntags;i++)
@@ -437,19 +451,25 @@ static int wayType_function(const char *_tag_,int _type_,const char *id,const ch
 
 static int relationType_function(const char *_tag_,int _type_,const char *id,const char *timestamp,const char *uid,const char *user,const char *visible,const char *version,const char *action)
 {
+ static int64_t llid;
+
  if(_type_&XMLPARSE_TAG_START)
    {
     nrelations++;
 
     if(!(nrelations%1000))
-       fprintf_middle(stderr,"Reading: Lines=%llu Nodes=%lu Ways=%lu Relations=%lu",ParseXML_LineNumber(),nnodes,nways,nrelations);
+       fprintf_middle(stderr,"Reading: Lines=%"PRIu64" Nodes=%"PRIu64" Ways=%"PRIu64" Relations=%"PRIu64,ParseXML_LineNumber(),nnodes,nways,nrelations);
 
     current_tags=NewTagList();
+
+    /* Handle the relation information */
+
+    XMLPARSE_ASSERT_INTEGER(_tag_,id); llid=atoll(id); /* need int64_t conversion */
    }
 
  if(_type_&XMLPARSE_TAG_END)
    {
-    TagList *result=ApplyTaggingRules(&RelationRules,current_tags);
+    TagList *result=ApplyRelationTaggingRules(current_tags,llid);
     int i;
 
     for(i=0;i<result->ntags;i++)
@@ -531,8 +551,9 @@ static int xmlDeclaration_function(const char *_tag_,int _type_,const char *vers
 
 int main(int argc,char **argv)
 {
- char *tagging=NULL,*filename=NULL;
- FILE *file;
+ char *tagging=NULL,*filename=NULL,*errorlog=NULL;
+ char *p;
+ int fd;
  int arg,retval;
 
  /* Parse the command line arguments */
@@ -540,21 +561,30 @@ int main(int argc,char **argv)
  for(arg=1;arg<argc;arg++)
    {
     if(!strcmp(argv[arg],"--help"))
-       print_usage(1);
-    else if(!strcmp(argv[arg],"--loggable"))
-       option_loggable=1;
+       print_usage(1,NULL,NULL);
     else if(!strncmp(argv[arg],"--tagging=",10))
        tagging=&argv[arg][10];
+    else if(!strcmp(argv[arg],"--loggable"))
+       option_loggable=1;
+    else if(!strcmp(argv[arg],"--logtime"))
+       option_logtime=1;
+    else if(!strcmp(argv[arg],"--errorlog"))
+       errorlog="error.log";
+    else if(!strncmp(argv[arg],"--errorlog=",11))
+       errorlog=&argv[arg][11];
     else if(argv[arg][0]=='-' && argv[arg][1]=='-')
-       print_usage(0);
+       print_usage(0,argv[arg],NULL);
     else if(filename)
-       print_usage(0);
+       print_usage(0,argv[arg],"Only one file name can be specified on the command line.");
     else
        filename=argv[arg];
    }
 
  /* Check the specified command line options */
 
+ if(!filename)
+    print_usage(0,NULL,"A single file name must be specified on the command line.");
+
  if(tagging)
    {
     if(!ExistsFile(tagging))
@@ -582,31 +612,35 @@ int main(int argc,char **argv)
 
  /* Open the input file */
 
- if(filename)
-   {
-    file=fopen(filename,"rb");
+ fd=ReOpenFile(filename);
 
-    if(!file)
-      {
-       fprintf(stderr,"Cannot open file '%s' for reading [%s].\n",argv[arg],strerror(errno));
-       exit(EXIT_FAILURE);
-      }
-   }
- else
-    file=stdin;
+ if((p=strstr(filename,".bz2")) && !strcmp(p,".bz2"))
+    fd=Uncompress_Bzip2(fd);
+
+ if((p=strstr(filename,".gz")) && !strcmp(p,".gz"))
+    fd=Uncompress_Gzip(fd);
+
+ /* Create the error log file */
+
+ if(errorlog)
+    open_errorlog(errorlog,0);
 
  /* Parse the file */
 
  fprintf_first(stderr,"Reading: Lines=0 Nodes=0 Ways=0 Relations=0");
 
- retval=ParseXML(file,xml_toplevel_tags,XMLPARSE_UNKNOWN_ATTR_IGNORE);
+ retval=ParseXML(fd,xml_toplevel_tags,XMLPARSE_UNKNOWN_ATTR_IGNORE);
 
- fprintf_last(stderr,"Read: Lines=%llu Nodes=%lu Ways=%lu Relations=%lu",ParseXML_LineNumber(),nnodes,nways,nrelations);
+ fprintf_last(stderr,"Read: Lines=%"PRIu64" Nodes=%"PRIu64" Ways=%"PRIu64" Relations=%"PRIu64,ParseXML_LineNumber(),nnodes,nways,nrelations);
+
+ /* Close the error log file */
+
+ if(errorlog)
+    close_errorlog();
 
  /* Tidy up */
 
- if(filename)
-    fclose(file);
+ CloseFile(fd);
 
  return(retval);
 }
@@ -616,28 +650,58 @@ int main(int argc,char **argv)
   Print out the usage information.
 
   int detail The level of detail to use - 0 = low, 1 = high.
+
+  const char *argerr The argument that gave the error (if there is one).
+
+  const char *err Other error message (if there is one).
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void print_usage(int detail)
+static void print_usage(int detail,const char *argerr,const char *err)
 {
  fprintf(stderr,
          "Usage: tagmodifier [--help]\n"
-         "                   [--loggable]\n"
          "                   [--tagging=<filename>]\n"
-         "                   [<filename.osm>]\n");
+         "                   [--loggable] [--logtime]\n"
+         "                   [--errorlog[=<name>]]\n"
+         "                   <filename.osm>"
+#if defined(USE_BZIP2) && USE_BZIP2
+         " | <filename.osm.bz2>"
+#endif
+#if defined(USE_GZIP) && USE_GZIP
+         " | <filename.osm.gz>"
+#endif
+         "\n");
+
+ if(argerr)
+    fprintf(stderr,
+            "\n"
+            "Error with command line parameter: %s\n",argerr);
+
+ if(err)
+    fprintf(stderr,
+            "\n"
+            "Error: %s\n",err);
 
  if(detail)
     fprintf(stderr,
             "\n"
             "--help                    Prints this information.\n"
             "\n"
-            "--loggable                Print progress messages suitable for logging to file.\n"
-            "\n"
             "--tagging=<filename>      The name of the XML file containing the tagging rules\n"
             "                          (defaults to 'tagging.xml' in current directory).\n"
             "\n"
-            "<filename.osm>            The name of the file to process (by default data is\n"
-            "                          read from standard input).\n");
+            "--loggable                Print progress messages suitable for logging to file.\n"
+            "--logtime                 Print the elapsed time for the processing.\n"
+            "--errorlog[=<name>]       Log parsing errors to 'error.log' or the given name.\n"
+            "\n"
+            "<filename.osm>            The name of the file to process.\n"
+#if defined(USE_BZIP2) && USE_BZIP2
+            "                          Filenames ending '.bz2' will be bzip2 uncompressed.\n"
+#endif
+#if defined(USE_GZIP) && USE_GZIP
+            "                          Filenames ending '.gz' will be gzip uncompressed.\n"
+#endif
+            );
 
  exit(!detail);
 }
