@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2012 Andrew M. Bishop
+ This file Copyright 2008-2013 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -61,8 +61,9 @@ static double LonMin;
 static double LonMax;
 
 static int limit_type=0;
-static Highway highways=Way_Count;
+static Highway highways=Highway_None;
 static Transports transports=Transports_None;
+static Properties properties=Properties_None;
 
 /* Local functions */
 
@@ -75,6 +76,7 @@ static void output_transport(index_t node,double latitude,double longitude);
 static void output_barrier(index_t node,double latitude,double longitude);
 static void output_turnrestriction(index_t node,double latitude,double longitude);
 static void output_limits(index_t node,double latitude,double longitude);
+static void output_property(index_t node,double latitude,double longitude);
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -130,26 +132,26 @@ void OutputJunctions(Nodes *nodes,Segments *segments,Ways *ways,Relations *relat
 static void output_junctions(index_t node,double latitude,double longitude)
 {
  Node *nodep=LookupNode(OSMNodes,node,1);
- Segment *segment;
- Way *firstway;
+ Segment *segmentp;
+ Way *firstwayp;
  int count=0,difference=0;
 
- segment=FirstSegment(OSMSegments,nodep,1);
- firstway=LookupWay(OSMWays,segment->way,1);
+ segmentp=FirstSegment(OSMSegments,nodep,1);
+ firstwayp=LookupWay(OSMWays,segmentp->way,1);
 
  do
    {
-    Way *way=LookupWay(OSMWays,segment->way,2);
+    Way *wayp=LookupWay(OSMWays,segmentp->way,2);
 
-    if(IsNormalSegment(segment))
+    if(IsNormalSegment(segmentp))
        count++;
 
-    if(WaysCompare(firstway,way))
+    if(WaysCompare(firstwayp,wayp))
        difference=1;
 
-    segment=NextSegment(OSMSegments,segment,node);
+    segmentp=NextSegment(OSMSegments,segmentp,node);
    }
- while(segment);
+ while(segmentp);
 
  if(count!=2 || difference)
     printf("%.6f %.6f %d\n",radians_to_degrees(latitude),radians_to_degrees(longitude),count);
@@ -209,20 +211,20 @@ void OutputSuper(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations
 static void output_super(index_t node,double latitude,double longitude)
 {
  Node *nodep=LookupNode(OSMNodes,node,1);
- Segment *segment;
+ Segment *segmentp;
 
  if(!IsSuperNode(nodep))
     return;
 
  printf("%.6f %.6f n\n",radians_to_degrees(latitude),radians_to_degrees(longitude));
 
- segment=FirstSegment(OSMSegments,nodep,1);
+ segmentp=FirstSegment(OSMSegments,nodep,1);
 
  do
    {
-    if(IsSuperSegment(segment))
+    if(IsSuperSegment(segmentp))
       {
-       index_t othernode=OtherNode(segment,node);
+       index_t othernode=OtherNode(segmentp,node);
        double lat,lon;
 
        GetLatLong(OSMNodes,othernode,&lat,&lon);
@@ -231,9 +233,9 @@ static void output_super(index_t node,double latitude,double longitude)
           printf("%.6f %.6f s\n",radians_to_degrees(lat),radians_to_degrees(lon));
       }
 
-    segment=NextSegment(OSMSegments,segment,node);
+    segmentp=NextSegment(OSMSegments,segmentp,node);
    }
- while(segment);
+ while(segmentp);
 }
 
 
@@ -290,15 +292,15 @@ void OutputOneway(Nodes *nodes,Segments *segments,Ways *ways,Relations *relation
 static void output_oneway(index_t node,double latitude,double longitude)
 {
  Node *nodep=LookupNode(OSMNodes,node,1);
- Segment *segment;
+ Segment *segmentp;
 
- segment=FirstSegment(OSMSegments,nodep,1);
+ segmentp=FirstSegment(OSMSegments,nodep,1);
 
  do
    {
-    if(IsNormalSegment(segment))
+    if(IsNormalSegment(segmentp))
       {
-       index_t othernode=OtherNode(segment,node);
+       index_t othernode=OtherNode(segmentp,node);
 
        if(node>othernode)
          {
@@ -306,16 +308,16 @@ static void output_oneway(index_t node,double latitude,double longitude)
 
           GetLatLong(OSMNodes,othernode,&lat,&lon);
 
-          if(IsOnewayFrom(segment,node))
+          if(IsOnewayFrom(segmentp,node))
              printf("%.6f %.6f %.6f %.6f\n",radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
-          else if(IsOnewayFrom(segment,othernode))
+          else if(IsOnewayFrom(segmentp,othernode))
              printf("%.6f %.6f %.6f %.6f\n",radians_to_degrees(lat),radians_to_degrees(lon),radians_to_degrees(latitude),radians_to_degrees(longitude));
          }
       }
 
-    segment=NextSegment(OSMSegments,segment,node);
+    segmentp=NextSegment(OSMSegments,segmentp,node);
    }
- while(segment);
+ while(segmentp);
 }
 
 
@@ -376,21 +378,21 @@ void OutputHighway(Nodes *nodes,Segments *segments,Ways *ways,Relations *relatio
 static void output_highway(index_t node,double latitude,double longitude)
 {
  Node *nodep=LookupNode(OSMNodes,node,1);
- Segment *segment;
+ Segment *segmentp;
 
- segment=FirstSegment(OSMSegments,nodep,1);
+ segmentp=FirstSegment(OSMSegments,nodep,1);
 
  do
    {
-    if(IsNormalSegment(segment))
+    if(IsNormalSegment(segmentp))
       {
-       index_t othernode=OtherNode(segment,node);
+       index_t othernode=OtherNode(segmentp,node);
 
        if(node>othernode)
          {
-          Way *way=LookupWay(OSMWays,segment->way,1);
+          Way *wayp=LookupWay(OSMWays,segmentp->way,1);
 
-          if(HIGHWAY(way->type)==highways)
+          if(HIGHWAY(wayp->type)==highways)
             {
              double lat,lon;
 
@@ -401,14 +403,14 @@ static void output_highway(index_t node,double latitude,double longitude)
          }
       }
 
-    segment=NextSegment(OSMSegments,segment,node);
+    segmentp=NextSegment(OSMSegments,segmentp,node);
    }
- while(segment);
+ while(segmentp);
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for segments allowed for a paticular type of traffic.
+  Output the data for segments allowed for a particular type of traffic.
 
   Nodes *nodes The set of nodes to use.
 
@@ -464,21 +466,21 @@ void OutputTransport(Nodes *nodes,Segments *segments,Ways *ways,Relations *relat
 static void output_transport(index_t node,double latitude,double longitude)
 {
  Node *nodep=LookupNode(OSMNodes,node,1);
- Segment *segment;
+ Segment *segmentp;
 
- segment=FirstSegment(OSMSegments,nodep,1);
+ segmentp=FirstSegment(OSMSegments,nodep,1);
 
  do
    {
-    if(IsNormalSegment(segment))
+    if(IsNormalSegment(segmentp))
       {
-       index_t othernode=OtherNode(segment,node);
+       index_t othernode=OtherNode(segmentp,node);
 
        if(node>othernode)
          {
-          Way *way=LookupWay(OSMWays,segment->way,1);
+          Way *wayp=LookupWay(OSMWays,segmentp->way,1);
 
-          if(way->allow&transports)
+          if(wayp->allow&transports)
             {
              double lat,lon;
 
@@ -489,14 +491,14 @@ static void output_transport(index_t node,double latitude,double longitude)
          }
       }
 
-    segment=NextSegment(OSMSegments,segment,node);
+    segmentp=NextSegment(OSMSegments,segmentp,node);
    }
- while(segment);
+ while(segmentp);
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Output the data for nodes disallowed for a paticular type of traffic.
+  Output the data for nodes disallowed for a particular type of traffic.
 
   Nodes *nodes The set of nodes to use.
 
@@ -621,17 +623,17 @@ static void output_turnrestriction(index_t node,double latitude,double longitude
  do
    {
     TurnRelation *relation;
-    Segment *from_segment,*to_segment;
+    Segment *from_segmentp,*to_segmentp;
     index_t from_node,to_node;
     double from_lat,from_lon,to_lat,to_lon;
 
     relation=LookupTurnRelation(OSMRelations,turnrelation,1);
 
-    from_segment=LookupSegment(OSMSegments,relation->from,1);
-    to_segment  =LookupSegment(OSMSegments,relation->to  ,2);
+    from_segmentp=LookupSegment(OSMSegments,relation->from,1);
+    to_segmentp  =LookupSegment(OSMSegments,relation->to  ,2);
 
-    from_node=OtherNode(from_segment,node);
-    to_node=OtherNode(to_segment,node);
+    from_node=OtherNode(from_segmentp,node);
+    to_node=OtherNode(to_segmentp,node);
 
     GetLatLong(OSMNodes,from_node,&from_lat,&from_lon);
     GetLatLong(OSMNodes,to_node,&to_lat,&to_lon);
@@ -869,37 +871,37 @@ void OutputLengthLimits(Nodes *nodes,Segments *segments,Ways *ways,Relations *re
 static void output_limits(index_t node,double latitude,double longitude)
 {
  Node *nodep=LookupNode(OSMNodes,node,1);
- Segment *segment,segments[MAX_SEG_PER_NODE];
+ Segment *segmentp,segmentps[MAX_SEG_PER_NODE];
  int limits[MAX_SEG_PER_NODE];
  int count=0;
  int i,j,same=0;
 
- segment=FirstSegment(OSMSegments,nodep,1);
+ segmentp=FirstSegment(OSMSegments,nodep,1);
 
  do
    {
-    if(IsNormalSegment(segment) && count<MAX_SEG_PER_NODE)
+    if(IsNormalSegment(segmentp) && count<MAX_SEG_PER_NODE)
       {
-       Way *way=LookupWay(OSMWays,segment->way,1);
+       Way *wayp=LookupWay(OSMWays,segmentp->way,1);
 
-       segments[count]=*segment;
+       segmentps[count]=*segmentp;
 
        switch(limit_type)
          {
-         case SPEED_LIMIT:  limits[count]=way->speed;  break;
-         case WEIGHT_LIMIT: limits[count]=way->weight; break;
-         case HEIGHT_LIMIT: limits[count]=way->height; break;
-         case WIDTH_LIMIT:  limits[count]=way->width;  break;
-         case LENGTH_LIMIT: limits[count]=way->length; break;
+         case SPEED_LIMIT:  limits[count]=wayp->speed;  break;
+         case WEIGHT_LIMIT: limits[count]=wayp->weight; break;
+         case HEIGHT_LIMIT: limits[count]=wayp->height; break;
+         case WIDTH_LIMIT:  limits[count]=wayp->width;  break;
+         case LENGTH_LIMIT: limits[count]=wayp->length; break;
          }
 
-       if(limits[count] || HIGHWAY(way->type)<Way_Track)
+       if(limits[count] || HIGHWAY(wayp->type)<Highway_Track)
           count++;
       }
 
-    segment=NextSegment(OSMSegments,segment,node);
+    segmentp=NextSegment(OSMSegments,segmentp,node);
    }
- while(segment);
+ while(segmentp);
 
  /* Nodes with only one limit are not interesting */
 
@@ -931,7 +933,7 @@ static void output_limits(index_t node,double latitude,double longitude)
       {
        double lat,lon;
 
-       GetLatLong(OSMNodes,OtherNode(&segments[i],node),&lat,&lon);
+       GetLatLong(OSMNodes,OtherNode(&segmentps[i],node),&lat,&lon);
 
        switch(limit_type)
          {
@@ -956,6 +958,94 @@ static void output_limits(index_t node,double latitude,double longitude)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Output the data for segments that have a particular property.
+
+  Nodes *nodes The set of nodes to use.
+
+  Segments *segments The set of segments to use.
+
+  Ways *ways The set of ways to use.
+
+  Relations *relations The set of relations to use.
+
+  double latmin The minimum latitude.
+
+  double latmax The maximum latitude.
+
+  double lonmin The minimum longitude.
+
+  double lonmax The maximum longitude.
+
+  Property property The type of property.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void OutputProperty(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,double latmin,double latmax,double lonmin,double lonmax,Property property)
+{
+ /* Use local variables so that the callback doesn't need to pass them backwards and forwards */
+
+ OSMNodes=nodes;
+ OSMSegments=segments;
+ OSMWays=ways;
+ OSMRelations=relations;
+
+ LatMin=latmin;
+ LatMax=latmax;
+ LonMin=lonmin;
+ LonMax=lonmax;
+
+ /* Iterate through the nodes and process them */
+
+ properties=PROPERTIES(property);
+
+ find_all_nodes(nodes,(callback_t)output_property);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Process a single node and all connected one-way segments (called as a callback).
+
+  index_t node The node to output.
+
+  double latitude The latitude of the node.
+
+  double longitude The longitude of the node.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void output_property(index_t node,double latitude,double longitude)
+{
+ Node *nodep=LookupNode(OSMNodes,node,1);
+ Segment *segmentp;
+
+ segmentp=FirstSegment(OSMSegments,nodep,1);
+
+ do
+   {
+    if(IsNormalSegment(segmentp))
+      {
+       index_t othernode=OtherNode(segmentp,node);
+
+       if(node>othernode)
+         {
+          Way *wayp=LookupWay(OSMWays,segmentp->way,1);
+
+          if(wayp->props&properties)
+            {
+             double lat,lon;
+
+             GetLatLong(OSMNodes,othernode,&lat,&lon);
+
+             printf("%.6f %.6f %.6f %.6f\n",radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
+            }
+         }
+      }
+
+    segmentp=NextSegment(OSMSegments,segmentp,node);
+   }
+ while(segmentp);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   A function to iterate through all nodes and call a callback function for each one.
 
@@ -988,10 +1078,10 @@ static void find_all_nodes(Nodes *nodes,callback_t callback)
 
        for(i=index1;i<index2;i++)
          {
-          Node *node=LookupNode(nodes,i,1);
+          Node *nodep=LookupNode(nodes,i,1);
 
-          double lat=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb)+off_to_latlong(node->latoffset));
-          double lon=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb)+off_to_latlong(node->lonoffset));
+          double lat=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb)+off_to_latlong(nodep->latoffset));
+          double lon=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb)+off_to_latlong(nodep->lonoffset));
 
           if(lat>LatMin && lat<LatMax && lon>LonMin && lon<LonMax)
              (*callback)(i,lat,lon);
