@@ -3,7 +3,7 @@
 //
 // Part of the Routino routing software.
 //
-// This file Copyright 2008-2012 Andrew M. Bishop
+// This file Copyright 2008-2013 Andrew M. Bishop
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Affero General Public License as published by
@@ -59,6 +59,8 @@ for(var marker=1;marker<=mapprops.maxmarkers;marker++)
    routino.point[marker].lat="";
    routino.point[marker].search="";
    routino.point[marker].active=false;
+   routino.point[marker].used=false;
+   routino.point[marker].home=false;
   }
 
 // Process the URL query string and extract the arguments
@@ -113,7 +115,7 @@ if(location.search.length>1)
 // Fill in the HTML - add the missing waypoints
 //
 
-function html_init()
+function html_init()            // called from router.html
 {
  var waypoints=document.getElementById("waypoints");
 
@@ -127,16 +129,16 @@ function html_init()
    {
     var searchresults=waypoints.insertRow(0);
 
+    searchresults.style.display="none";
     searchresults.id="searchresults" + marker;
     searchresults.innerHTML=searchresults_html.split('XXX').join(marker);
 
     var waypoint=waypoints.insertRow(0);
 
+    waypoint.style.display="none";
     waypoint.id="waypoint" + marker;
     waypoint.innerHTML=waypoint_html.split('XXX').join(marker);
    }
-
- vismarkers=mapprops.maxmarkers;
 }
 
 
@@ -148,11 +150,11 @@ function html_init()
 // Form initialisation - fill in the uninitialised parts
 //
 
-function form_init()
+function form_init()            // called from router.html
 {
  // Fill in the waypoints
 
- var filled=0;
+ vismarkers=0;
 
  for(var marker=mapprops.maxmarkers;marker>=1;marker--)
    {
@@ -162,33 +164,47 @@ function form_init()
 
     if(lon != undefined && lat != undefined && search != undefined && lon != "" && lat != "" && search != "")
       {
+       markerAddForm(marker);
+
        formSetSearch(marker,search);
-       formSetCoords(marker,lon,lat,true);
+       formSetCoords(marker,lon,lat);
+
+       markerAddMap(marker);
 
        markerSearch(marker);
 
-       filled++;
+       vismarkers++;
       }
     else if(lon != undefined && lat != undefined && lon != "" && lat != "")
       {
-       formSetCoords(marker,lon,lat,true);
+       markerAddForm(marker);
+
+       formSetCoords(marker,lon,lat);
+
+       markerAddMap(marker);
 
        markerCoords(marker);
 
-       filled++;
+       vismarkers++;
       }
     else if(search != undefined && search != "")
       {
+       markerAddForm(marker);
+
        formSetSearch(marker,search);
 
        markerSearch(marker);
 
        DoSearch(marker);
 
-       filled++;
+       vismarkers++;
+      }
+    else if(vismarkers || marker<=2)
+      {
+       markerAddForm(marker);
+
+       vismarkers++;
       }
-    else if(filled==0)
-       markerRemove(marker);
 
     var searchfield=document.forms["form"].elements["search" + marker];
 
@@ -253,19 +269,13 @@ function form_init()
 
  if(homelon!=null && homelat!=null)
    {
-    for(var marker=mapprops.maxmarkers;marker>=1;marker--)
-      {
-       var lon=routino.point[marker].lon;
-       var lat=routino.point[marker].lat;
-
-       if(lon==homelon && lat==homelat)
-          updateIcon(marker);
-      }
+    for(var m=1;m<=vismarkers;m++)
+       markerCheckHome(m);
 
     // If the first location is empty and the cookie is set then fill it.
 
-    if(routino.point[1].lon=="" && routino.point[1].lat=="")
-       formSetCoords(1,homelon,homelat,true);
+    if(!routino.point[1].used)
+       markerMoveHome(1);
    }
 }
 
@@ -289,7 +299,7 @@ function searchOnReturnKey(ev)
 // Change of language in the form
 //
 
-function formSetLanguage(value)
+function formSetLanguage(value) // called from router.html (with no arguments)
 {
  if(value == undefined)
    {
@@ -314,7 +324,7 @@ function formSetLanguage(value)
 // Change of transport in the form
 //
 
-function formSetTransport(value)
+function formSetTransport(value) // called from router.html
 {
  routino.transport=value;
 
@@ -346,7 +356,7 @@ function formSetTransport(value)
 // Change of highway in the form
 //
 
-function formSetHighway(type,value)
+function formSetHighway(type,value) // called from router.html (with one argument)
 {
  if(value == undefined)
     routino.profile_highway[type][routino.transport]=document.forms["form"].elements["highway-" + type].value;
@@ -364,7 +374,7 @@ function formSetHighway(type,value)
 // Change of Speed in the form
 //
 
-function formSetSpeed(type,value)
+function formSetSpeed(type,value) // called from router.html (with one argument)
 {
  if(value == undefined)
     routino.profile_speed[type][routino.transport]=document.forms["form"].elements["speed-" + type].value;
@@ -382,7 +392,7 @@ function formSetSpeed(type,value)
 // Change of Property in the form
 //
 
-function formSetProperty(type,value)
+function formSetProperty(type,value) // called from router.html (with one argument)
 {
  if(value == undefined)
     routino.profile_property[type][routino.transport]=document.forms["form"].elements["property-" + type].value;
@@ -400,7 +410,7 @@ function formSetProperty(type,value)
 // Change of Restriction rule in the form
 //
 
-function formSetRestriction(type,value)
+function formSetRestriction(type,value) // called from router.html (with one argument)
 {
  if(value == undefined)
    {
@@ -427,66 +437,72 @@ function formSetRestriction(type,value)
 // Set the feature coordinates from the form when the form changes.
 //
 
-function formSetCoords(marker,lon,lat,active)
+function formSetCoords(marker,lon,lat) // called from router.html (with one argument)
 {
  clearSearchResult(marker);
 
- if(lon == undefined || lat == undefined)
+ if(lon == undefined && lat == undefined)
    {
-    routino.point[marker].lon=document.forms["form"].elements["lon" + marker].value;
-    routino.point[marker].lat=document.forms["form"].elements["lat" + marker].value;
+    lon=document.forms["form"].elements["lon" + marker].value;
+    lat=document.forms["form"].elements["lat" + marker].value;
+   }
 
-    if(routino.point[marker].lon=="" || routino.point[marker].lat=="")
-       markerCentre(marker);
+ if(lon == "" && lat == "")
+   {
+    document.forms["form"].elements["lon" + marker].value="";
+    document.forms["form"].elements["lat" + marker].value="";
+
+    routino.point[marker].lon="";
+    routino.point[marker].lat=""
    }
  else
    {
+    if(lon=="")
+      {
+       var lonlat=map.getCenter().clone();
+       lonlat.transform(epsg900913,epsg4326);
+
+       lon=lonlat.lon;
+      }
+
+    if(lon<-180) lon=-180;
+    if(lon>+180) lon=+180;
+
+    if(lat=="")
+      {
+       var lonlat=map.getCenter().clone();
+       lonlat.transform(epsg900913,epsg4326);
+
+       lat=lonlat.lat;
+      }
+
+    if(lat<-90 ) lat=-90 ;
+    if(lat>+90 ) lat=+90 ;
+
+    var lonlat = new OpenLayers.LonLat(lon,lat);
+    lonlat.transform(epsg4326,epsg900913);
+
+    markers[marker].move(lonlat);
+
+    markersmoved=true;
+
     document.forms["form"].elements["lon" + marker].value=format5f(lon);
     document.forms["form"].elements["lat" + marker].value=format5f(lat);
 
     routino.point[marker].lon=lon;
     routino.point[marker].lat=lat;
+    routino.point[marker].used=true;
 
-    if(active != undefined)
-      {
-       if(active)
-          markerAddMap(marker);
-       else
-          markerRemoveMap(marker);
-      }
+    markerCheckHome(marker);
    }
-
- var lonlat=map.getCenter().clone();
- lonlat.transform(epsg900913,epsg4326);
-
- if(routino.point[marker].lon!="")
-   {
-    if(routino.point[marker].lon<-180) routino.point[marker].lon=-180;
-    if(routino.point[marker].lon>+180) routino.point[marker].lon=+180;
-    lonlat.lon=routino.point[marker].lon;
-   }
-
- if(routino.point[marker].lat!="")
-   {
-    if(routino.point[marker].lat<-90 ) routino.point[marker].lat=-90 ;
-    if(routino.point[marker].lat>+90 ) routino.point[marker].lat=+90 ;
-    lonlat.lat=routino.point[marker].lat;
-   }
-
- lonlat=lonlat.clone()
- lonlat.transform(epsg4326,epsg900913);
-
- markers[marker].move(lonlat);
-
- markersmoved=true;
 }
 
 
 //
-// Set the feature coordinates from the form when the form changes.
+// Set the search field from the form when the form changes.
 //
 
-function formSetSearch(marker,search)
+function formSetSearch(marker,search) // called from event handler linked to router.html (with one argument)
 {
  clearSearchResult(marker);
 
@@ -588,7 +604,7 @@ function buildMapArguments()
 // Update a URL
 //
 
-function updateURL(element)
+function updateURL(element)     // called from router.html
 {
  if(element.id == "permalink_url")
     element.href=location.pathname + "?" + buildURLArguments(true) + ";" + buildMapArguments();
@@ -616,7 +632,7 @@ var epsg4326, epsg900913;
 // Initialise the 'map' object
 //
 
-function map_init()
+function map_init()             // called from router.html
 {
  lon =args["lon"];
  lat =args["lat"];
@@ -660,10 +676,8 @@ function map_init()
  for(var l=0;l < mapprops.mapdata.length;l++)
    {
     layerMap[l] = new OpenLayers.Layer.TMS(mapprops.mapdata[l].label,
-                                           mapprops.mapdata[l].baseurl,
+                                           mapprops.mapdata[l].tileurl,
                                            {
-                                            emptyUrl: mapprops.mapdata[l].errorurl,
-                                            type: 'png',
                                             getURL: limitedUrl,
                                             displayOutsideMaxExtent: true,
                                             buffer: 1
@@ -671,29 +685,54 @@ function map_init()
     map.addLayer(layerMap[l]);
    }
 
- // Get a URL for the tile; limited to map restricted extent.
+ // Update the attribution if the layer changes
+
+ map.events.register("changelayer",layerMap,change_attribution_event);
+
+ function change_attribution_event(event)
+ {
+  for(var l=0;l < mapprops.mapdata.length;l++)
+     if(this[l] == event.layer)
+        change_attribution(l);
+ }
+
+ function change_attribution(l)
+ {
+  var data_url =mapprops.mapdata[l].attribution.data_url;
+  var data_text=mapprops.mapdata[l].attribution.data_text;
+  var tile_url =mapprops.mapdata[l].attribution.tile_url;
+  var tile_text=mapprops.mapdata[l].attribution.tile_text;
+
+  document.getElementById("attribution_data").innerHTML="<a href=\"" + data_url + "\" target=\"data_attribution\">" + data_text + "</a>";
+  document.getElementById("attribution_tile").innerHTML="<a href=\"" + tile_url + "\" target=\"tile_attribution\">" + tile_text + "</a>";
+ }
+
+ change_attribution(0);
+
+ // Get a URL for the tile (mostly copied from OpenLayers/Layer/XYZ.js).
 
  function limitedUrl(bounds)
  {
-  var z = map.getZoom() + map.minZoomLevel;
-
-  if (z>=7 && (bounds.right  < map.restrictedExtent.left ||
-               bounds.left   > map.restrictedExtent.right ||
-               bounds.top    < map.restrictedExtent.bottom ||
-               bounds.bottom > map.restrictedExtent.top))
-     return this.emptyUrl;
-
   var res = map.getResolution();
-  var y = Math.round((this.maxExtent.top - bounds.top) / (res * this.tileSize.h));
-  var limit = Math.pow(2, z);
-
-  if (y < 0 || y >= limit)
-    return this.emptyUrl;
+  var res = this.getServerResolution();
 
   var x = Math.round((bounds.left - this.maxExtent.left) / (res * this.tileSize.w));
+  var y = Math.round((this.maxExtent.top - bounds.top) / (res * this.tileSize.h));
+  var z = map.getZoom() + map.minZoomLevel;
 
+  var limit = Math.pow(2, z);
   x = ((x % limit) + limit) % limit;
-  return this.url + z + "/" + x + "/" + y + "." + this.type;
+
+  var xyz = {'x': x, 'y': y, 'z': z};
+  var url = this.url;
+
+  if (OpenLayers.Util.isArray(url))
+    {
+     var s = '' + xyz.x + xyz.y + xyz.z;
+     url = this.selectUrl(s, url);
+    }
+        
+  return OpenLayers.String.format(url, xyz);
  }
 
  // Define a GPX layer but don't add it yet
@@ -789,11 +828,7 @@ function dragMove(feature,pixel)
 {
  for(var marker in markers)
     if(feature==markers[marker])
-      {
-       markersmoved=true;
-
        dragSetForm(marker);
-      }
 }
 
 
@@ -805,11 +840,7 @@ function dragComplete(feature,pixel)
 {
  for(var marker in markers)
     if(feature==markers[marker])
-      {
-       markersmoved=true;
-
        dragSetForm(marker);
-      }
 }
 
 
@@ -838,14 +869,29 @@ function dragSetForm(marker)
 // Toggle a marker on the map.
 //
 
-function markerToggleMap(marker)
+function markerToggleMap(marker) // called from router.html
 {
- clearSearchResult(marker);
+ if(!routino.point[marker].used)
+   {
+    routino.point[marker].used=true;
+    markerCentre(marker);
+    markerCoords(marker);
+   }
 
- if(routino.point[marker].active)
-    markerRemoveMap(marker);
- else
+ markerAddRemoveMap(marker,!routino.point[marker].active);
+}
+
+
+//
+// Show or hide a marker on the map.
+//
+
+function markerAddRemoveMap(marker,active)
+{
+ if(active)
     markerAddMap(marker);
+ else
+    markerRemoveMap(marker);
 }
 
 
@@ -859,6 +905,7 @@ function markerAddMap(marker)
 
  markers[marker].style.display = "";
  routino.point[marker].active=true;
+ routino.point[marker].used=true;
 
  updateIcon(marker);
 
@@ -887,7 +934,7 @@ function markerRemoveMap(marker)
 // Display search string for the marker
 //
 
-function markerSearch(marker)
+function markerSearch(marker)   // called from router.html
 {
  clearSearchResult(marker);
 
@@ -900,7 +947,7 @@ function markerSearch(marker)
 // Display coordinates for the marker
 //
 
-function markerCoords(marker)
+function markerCoords(marker)   // called from router.html
 {
  clearSearchResult(marker);
 
@@ -913,14 +960,17 @@ function markerCoords(marker)
 // Centre the marker on the map
 //
 
-function markerCentre(marker)
+function markerCentre(marker)   // called from router.html
 {
+ if(!routino.point[marker].used)
+    return;
+
  clearSearchResult(marker);
 
  var lonlat=map.getCenter().clone();
  lonlat.transform(epsg900913,epsg4326);
 
- formSetCoords(marker,lonlat.lon,lonlat.lat,true);
+ formSetCoords(marker,lonlat.lon,lonlat.lat);
 }
 
 
@@ -928,8 +978,11 @@ function markerCentre(marker)
 // Centre the map on the marker
 //
 
-function markerRecentre(marker)
+function markerRecentre(marker) // called from router.html
 {
+ if(!routino.point[marker].used)
+    return;
+
  clearSearchResult(marker);
 
  lon=routino.point[marker].lon;
@@ -946,18 +999,14 @@ function markerRecentre(marker)
 // Clear the current marker.
 //
 
-function markerRemove(marker)
+function markerRemove(marker)   // called from router.html
 {
  clearSearchResult(marker);
 
- for(var marker2=marker;marker2<vismarkers;marker2++)
-    formSetCoords(marker2,routino.point[marker2+1].lon,routino.point[marker2+1].lat,routino.point[marker2+1].active);
+ for(var m=marker;m<vismarkers;m++)
+    markerCopy(m,m+1);
 
- markerRemoveMap(vismarkers);
-
- document.getElementById("waypoint" + vismarkers).style.display="none";
-
- vismarkers--;
+ markerRemoveForm(vismarkers--);
 
  if(vismarkers==1)
     markerAddAfter(1);
@@ -975,16 +1024,12 @@ function markerAddBefore(marker)
  if(vismarkers==mapprops.maxmarkers || marker==1)
     return false;
 
- vismarkers++;
+ markerAddForm(++vismarkers);
 
- document.getElementById("waypoint" + vismarkers).style.display="";
+ for(var m=vismarkers;m>marker;m--)
+    markerCopy(m,m-1);
 
- for(var marker2=vismarkers;marker2>marker;marker2--)
-    formSetCoords(marker2,routino.point[marker2-1].lon,routino.point[marker2-1].lat,routino.point[marker2-1].active);
-
- formSetCoords(marker,"","",false);
-
- markerRemoveMap(marker);
+ markerClearForm(marker-1);
 }
 
 
@@ -992,23 +1037,19 @@ function markerAddBefore(marker)
 // Add a marker after the current one.
 //
 
-function markerAddAfter(marker)
+function markerAddAfter(marker) // called from router.html
 {
  clearSearchResult(marker);
 
  if(vismarkers==mapprops.maxmarkers)
     return false;
 
- vismarkers++;
+ markerAddForm(++vismarkers);
 
- document.getElementById("waypoint" + vismarkers).style.display="";
+ for(var m=vismarkers;m>(marker+1);m--)
+    markerCopy(m,m-1);
 
- for(var marker2=vismarkers;marker2>(marker+1);marker2--)
-    formSetCoords(marker2,routino.point[marker2-1].lon,routino.point[marker2-1].lat,routino.point[marker2-1].active);
-
- formSetCoords(marker+1,"","",false);
-
- markerRemoveMap(marker+1);
+ markerClearForm(marker+1);
 }
 
 
@@ -1016,13 +1057,18 @@ function markerAddAfter(marker)
 // Set this marker as the home location.
 //
 
-function markerHome(marker)
+function markerHome(marker)     // called from router.html
 {
- clearSearchResult(marker);
+ if(!routino.point[marker].used)
+   {
+    markerMoveHome(marker);
+   }
+ else
+   {
+    clearSearchResult(marker);
 
- if(markerHomeCookie(marker))
-    for(marker=1;marker<=mapprops.maxmarkers;marker++)
-       updateIcon(marker);
+    markerSetClearHome(marker,!routino.point[marker].home);
+   }
 }
 
 
@@ -1030,14 +1076,15 @@ function markerHome(marker)
 // Set this marker as the current location.
 //
 
-function markerLocate(marker)
+function markerLocate(marker)   // called from router.html
 {
  clearSearchResult(marker);
 
  if(navigator.geolocation)
     navigator.geolocation.getCurrentPosition(
                                              function(position) {
-                                              formSetCoords(marker,position.coords.longitude,position.coords.latitude,true);
+                                              formSetCoords(marker,position.coords.longitude,position.coords.latitude);
+                                              markerAddMap(marker);
                                              });
 }
 
@@ -1048,10 +1095,7 @@ function markerLocate(marker)
 
 function updateIcon(marker)
 {
- var lon=routino.point[marker].lon;
- var lat=routino.point[marker].lat;
-
- if(lon==homelon && lat==homelat)
+ if(routino.point[marker].home)
    {
     if(routino.point[marker].active)
        document.images["waypoint" + marker].src="icons/marker-home-red.png";
@@ -1074,44 +1118,79 @@ function updateIcon(marker)
 }
 
 
+//
+// Move the marker to the home location
+//
+
+function markerMoveHome(marker)
+{
+ if(homelon==null || homelat==null)
+    return;
+
+ routino.point[marker].home=true;
+ routino.point[marker].used=true;
+
+ formSetCoords(marker,homelon,homelat);
+ markerAddMap(marker);
+}
+
+
 //
 // Set or clear the home marker icon
 //
 
-function markerHomeCookie(marker)
+function markerSetClearHome(marker,home)
 {
- var lon=routino.point[marker].lon;
- var lat=routino.point[marker].lat;
-
- if(lon=="" || lat=="")
-    return(false);
-
  var cookie;
  var date = new Date();
 
- if((homelat==null && homelon==null) ||
-    (homelat!=lat  && homelon!=lon))
+ if(home)
    {
-    cookie="Routino-home=lon:" + lon + ":lat:" + lat;
+    homelat=routino.point[marker].lat;
+    homelon=routino.point[marker].lon;
+
+    cookie="Routino-home=lon:" + homelon + ":lat:" + homelat;
 
     date.setUTCFullYear(date.getUTCFullYear()+5);
 
-    homelat=lat;
-    homelon=lon;
+    routino.point[marker].home=true;
    }
  else
    {
+    homelat=null;
+    homelon=null;
+
     cookie="Routino-home=unset";
 
     date.setUTCFullYear(date.getUTCFullYear()-1);
 
-    homelat=null;
-    homelon=null;
+    routino.point[marker].home=false;
    }
 
  document.cookie=cookie + ";expires=" + date.toGMTString();
 
- return(true);
+ updateIcon(marker);
+
+ for(m=1;m<=mapprops.maxmarkers;m++)
+    markerCheckHome(m);
+}
+
+
+//
+// Check if a marker is the home marker
+//
+
+function markerCheckHome(marker)
+{
+ var home=routino.point[marker].home;
+
+ if(routino.point[marker].lon==homelon && routino.point[marker].lat==homelat)
+    routino.point[marker].home=true;
+ else
+    routino.point[marker].home=false;
+
+ if(home!=routino.point[marker].home)
+    updateIcon(marker);
 }
 
 
@@ -1119,7 +1198,7 @@ function markerHomeCookie(marker)
 // Move this marker up.
 //
 
-function markerMoveUp(marker)
+function markerMoveUp(marker)   // called from router.html
 {
  if(marker==1)
    {
@@ -1135,7 +1214,7 @@ function markerMoveUp(marker)
 // Move this marker down.
 //
 
-function markerMoveDown(marker)
+function markerMoveDown(marker) // called from router.html
 {
  if(marker==vismarkers)
    {
@@ -1147,19 +1226,57 @@ function markerMoveDown(marker)
 }
 
 
+//
+// Copy a marker from one place to another.
+//
+
+function markerCopy(marker1,marker2)
+{
+ for(var element in routino.point[marker2])
+    routino.point[marker1][element]=routino.point[marker2][element];
+
+ document.getElementById("search" + marker1).style.display=document.getElementById("search" + marker2).style.display;
+
+ document.getElementById("coords" + marker1).style.display=document.getElementById("coords" + marker2).style.display;
+
+ document.forms["form"].elements["search" + marker1].value=document.forms["form"].elements["search" + marker2].value;
+
+ formSetCoords(marker1,routino.point[marker1].lon,routino.point[marker1].lat);
+
+ markerAddRemoveMap(marker1,routino.point[marker1].active);
+}
+
+
 //
 // Swap a pair of markers.
 //
 
 function markerSwap(marker1,marker2)
 {
- var lon=routino.point[marker1].lon;
- var lat=routino.point[marker1].lat;
- var active=routino.point[marker1].active;
+ for(var element in routino.point[marker2])
+   {
+    var temp=routino.point[marker1][element];
+    routino.point[marker1][element]=routino.point[marker2][element];
+    routino.point[marker2][element]=temp;
+   }
 
- formSetCoords(marker1,routino.point[marker2].lon,routino.point[marker2].lat,routino.point[marker2].active);
+ var search_display=document.getElementById("search" + marker1).style.display;
+ document.getElementById("search" + marker1).style.display=document.getElementById("search" + marker2).style.display;
+ document.getElementById("search" + marker2).style.display=search_display;
 
- formSetCoords(marker2,lon,lat,active);
+ var coords_display=document.getElementById("coords" + marker1).style.display;
+ document.getElementById("coords" + marker1).style.display=document.getElementById("coords" + marker2).style.display;
+ document.getElementById("coords" + marker2).style.display=coords_display;
+
+ var search_value=document.forms["form"].elements["search" + marker1].value;
+ document.forms["form"].elements["search" + marker1].value=document.forms["form"].elements["search" + marker2].value;
+ document.forms["form"].elements["search" + marker2].value=search_value;
+
+ formSetCoords(marker1,routino.point[marker1].lon,routino.point[marker1].lat);
+ formSetCoords(marker2,routino.point[marker2].lon,routino.point[marker2].lat);
+
+ markerAddRemoveMap(marker1,routino.point[marker1].active);
+ markerAddRemoveMap(marker2,routino.point[marker2].active);
 }
 
 
@@ -1167,13 +1284,73 @@ function markerSwap(marker1,marker2)
 // Reverse the markers.
 //
 
-function markersReverse()
+function markersReverse()       // called from router.html
 {
  for(var marker=1;marker<=vismarkers/2;marker++)
     markerSwap(marker,vismarkers+1-marker);
 }
 
 
+//
+// Close the loop.
+//
+
+function markersLoop()          // called from router.html
+{
+ if(vismarkers==mapprops.maxmarkers)
+    return false;
+
+ if(routino.point[vismarkers].lon==routino.point[1].lon && routino.point[vismarkers].lat==routino.point[1].lat)
+    return false;
+
+ if(routino.point[vismarkers].used)
+    markerAddForm(++vismarkers);
+
+ markerCopy(vismarkers,1);
+}
+
+
+//
+// Display the form for a marker
+//
+
+function markerAddForm(marker)
+{
+ document.getElementById("waypoint" + marker).style.display="";
+}
+
+
+//
+// Hide the form for a marker
+//
+
+function markerRemoveForm(marker)
+{
+ document.getElementById("waypoint" + marker).style.display="none";
+
+ markerClearForm(marker);
+}
+
+
+//
+// Clear the form for a marker
+//
+
+function markerClearForm(marker)
+{
+ markerRemoveMap(marker);
+ markerCoords(marker);
+
+ formSetCoords(marker,"","");
+ formSetSearch(marker,"");
+
+ updateIcon(marker);
+
+ routino.point[marker].used=false;
+ routino.point[marker].home=false;
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////
 //////////////////////////// Route results handling ////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////
@@ -1318,7 +1495,7 @@ function removeGPXTrace(type)
 // Display data statistics
 //
 
-function displayStatistics()
+function displayStatistics() // called from router.html
 {
  // Use AJAX to get the statistics
 
@@ -1341,7 +1518,7 @@ function runStatisticsSuccess(response)
 // Submit form - perform the routing
 //
 
-function findRoute(type)
+function findRoute(type) // called from router.html
 {
  tab_select("results");
 
@@ -1384,15 +1561,13 @@ function runRouterSuccess(response)
  var lines=response.responseText.split('\n');
 
  var uuid=lines[0];
- var cpuinfo=lines[1];  // not used
- var distinfo=lines[2]; // not used
- var message=lines[3];  // content not used
+ var success=lines[1];
 
  var link;
 
  // Update the status message
 
- if(message!="")
+ if(success=="ERROR")
    {
     displayStatus("result","error");
     hideshow_show('help_route');
@@ -1676,7 +1851,8 @@ function runSearchSuccess(response)
  if(searchresults[marker].length==1)
    {
     formSetSearch(marker,searchresults[marker][0].name);
-    formSetCoords(marker,searchresults[marker][0].lon,searchresults[marker][0].lat,true);
+    formSetCoords(marker,searchresults[marker][0].lon,searchresults[marker][0].lat);
+    markerAddMap(marker);
    }
  else
    {
@@ -1710,7 +1886,8 @@ function choseSearchResult(marker,n)
  if(n>=0)
    {
     formSetSearch(marker,searchresults[marker][n].name);
-    formSetCoords(marker,searchresults[marker][n].lon,searchresults[marker][n].lat,true);
+    formSetCoords(marker,searchresults[marker][n].lon,searchresults[marker][n].lat);
+    markerAddMap(marker);
    }
 }
 
