@@ -3,7 +3,7 @@
 //
 // Part of the Routino routing software.
 //
-// This file Copyright 2008-2012 Andrew M. Bishop
+// This file Copyright 2008-2013 Andrew M. Bishop
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Affero General Public License as published by
@@ -30,12 +30,14 @@ var data_types=[
                 "oneway",
                 "highway",
                 "transport",
+                "barrier",
                 "turns",
                 "speed",
                 "weight",
                 "height",
                 "width",
-                "length"
+                "length",
+                "property"
                ];
 
 
@@ -170,10 +172,8 @@ function map_init()             // called from visualiser.html
  for(var l=0;l < mapprops.mapdata.length;l++)
    {
     layerMap[l] = new OpenLayers.Layer.TMS(mapprops.mapdata[l].label,
-                                           mapprops.mapdata[l].baseurl,
+                                           mapprops.mapdata[l].tileurl,
                                            {
-                                            emptyUrl: mapprops.mapdata[l].errorurl,
-                                            type: 'png',
                                             getURL: limitedUrl,
                                             displayOutsideMaxExtent: true,
                                             buffer: 1
@@ -181,29 +181,54 @@ function map_init()             // called from visualiser.html
     map.addLayer(layerMap[l]);
    }
 
- // Get a URL for the tile; limited to map restricted extent.
+ // Update the attribution if the layer changes
+
+ map.events.register("changelayer",layerMap,change_attribution_event);
+
+ function change_attribution_event(event)
+ {
+  for(var l=0;l < mapprops.mapdata.length;l++)
+     if(this[l] == event.layer)
+        change_attribution(l);
+ }
+
+ function change_attribution(l)
+ {
+  var data_url =mapprops.mapdata[l].attribution.data_url;
+  var data_text=mapprops.mapdata[l].attribution.data_text;
+  var tile_url =mapprops.mapdata[l].attribution.tile_url;
+  var tile_text=mapprops.mapdata[l].attribution.tile_text;
+
+  document.getElementById("attribution_data").innerHTML="<a href=\"" + data_url + "\" target=\"data_attribution\">" + data_text + "</a>";
+  document.getElementById("attribution_tile").innerHTML="<a href=\"" + tile_url + "\" target=\"tile_attribution\">" + tile_text + "</a>";
+ }
+
+ change_attribution(0);
+
+ // Get a URL for the tile (mostly copied from OpenLayers/Layer/XYZ.js).
 
  function limitedUrl(bounds)
  {
-  var z = map.getZoom() + map.minZoomLevel;
-
-  if (z>=7 && (bounds.right  < map.restrictedExtent.left ||
-               bounds.left   > map.restrictedExtent.right ||
-               bounds.top    < map.restrictedExtent.bottom ||
-               bounds.bottom > map.restrictedExtent.top))
-     return this.emptyUrl;
-
   var res = map.getResolution();
-  var y = Math.round((this.maxExtent.top - bounds.top) / (res * this.tileSize.h));
-  var limit = Math.pow(2, z);
-
-  if (y < 0 || y >= limit)
-    return this.emptyUrl;
+  var res = this.getServerResolution();
 
   var x = Math.round((bounds.left - this.maxExtent.left) / (res * this.tileSize.w));
+  var y = Math.round((this.maxExtent.top - bounds.top) / (res * this.tileSize.h));
+  var z = map.getZoom() + map.minZoomLevel;
 
+  var limit = Math.pow(2, z);
   x = ((x % limit) + limit) % limit;
-  return this.url + z + "/" + x + "/" + y + "." + this.type;
+
+  var xyz = {'x': x, 'y': y, 'z': z};
+  var url = this.url;
+
+  if (OpenLayers.Util.isArray(url))
+    {
+     var s = '' + xyz.x + xyz.y + xyz.z;
+     url = this.selectUrl(s, url);
+    }
+        
+  return OpenLayers.String.format(url, xyz);
  }
 
  // Add a vectors layer
@@ -439,6 +464,14 @@ function displayData(datatype)  // called from visualiser.html
     url+="-" + transport;
     OpenLayers.Request.GET({url: url, success: runTransportSuccess, failure: runFailure});
     break;
+   case 'barrier':
+    var transports=document.forms["barriers"].elements["barrier"];
+    for(var t in transports)
+       if(transports[t].checked)
+          transport=transports[t].value;
+    url+="-" + transport;
+    OpenLayers.Request.GET({url: url, success: runBarrierSuccess, failure: runFailure});
+    break;
    case 'turns':
     OpenLayers.Request.GET({url: url, success: runTurnsSuccess, failure: runFailure});
     break;
@@ -449,6 +482,14 @@ function displayData(datatype)  // called from visualiser.html
    case 'length':
     OpenLayers.Request.GET({url: url, success: runLimitSuccess, failure: runFailure});
     break;
+   case 'property':
+    var properties=document.forms["properties"].elements["property"];
+    for(var p in properties)
+       if(properties[p].checked)
+          property=properties[p].value;
+    url+="-" + property;
+    OpenLayers.Request.GET({url: url, success: runPropertySuccess, falure: runFailure});
+    break;
    }
 }
 
@@ -731,6 +772,52 @@ function runTransportSuccess(response)
 }
 
 
+//
+// Success in getting the barrier data
+//
+
+function runBarrierSuccess(response)
+{
+ var lines=response.responseText.split('\n');
+
+ var features=[];
+
+ for(var line=0;line<lines.length;line++)
+   {
+    var words=lines[line].split(' ');
+
+    if(line == 0)
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
+
+       box = new OpenLayers.Marker.Box(bounds);
+
+       layerBoxes.addMarker(box);
+      }
+    else if(words[0] != "")
+      {
+       var lat=words[0];
+       var lon=words[1];
+
+       var lonlat= new OpenLayers.LonLat(lon,lat).transform(epsg4326,epsg900913);
+
+       var point = new OpenLayers.Geometry.Point(lonlat.lon,lonlat.lat);
+
+       features.push(new OpenLayers.Feature.Vector(point,{},super_node_style));
+      }
+   }
+
+ layerVectors.addFeatures(features);
+
+ displayStatus("data","barrier",lines.length-2);
+}
+
+
 //
 // Success in getting the turn restrictions data
 //
@@ -855,6 +942,58 @@ function runLimitSuccess(response)
 }
 
 
+//
+// Success in getting the property data
+//
+
+function runPropertySuccess(response)
+{
+ var lines=response.responseText.split('\n');
+
+ var features=[];
+
+ for(var line=0;line<lines.length;line++)
+   {
+    var words=lines[line].split(' ');
+
+    if(line == 0)
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
+
+       box = new OpenLayers.Marker.Box(bounds);
+
+       layerBoxes.addMarker(box);
+      }
+    else if(words[0] != "")
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
+       var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
+
+       var point1 = new OpenLayers.Geometry.Point(lonlat1.lon,lonlat1.lat);
+       var point2 = new OpenLayers.Geometry.Point(lonlat2.lon,lonlat2.lat);
+
+       var segment = new OpenLayers.Geometry.LineString([point1,point2]);
+
+       features.push(new OpenLayers.Feature.Vector(segment,{},super_segment_style));
+      }
+   }
+
+ layerVectors.addFeatures(features);
+
+ displayStatus("data","property",lines.length-2);
+}
+
+
 //
 // Failure in getting data.
 //
